//
//  PianoInstrument.m
//  PianoInstrument
//
//  Created by Chris Osborn on 7/1/15.
//  Copyright (c) 2015 Google. All rights reserved.
//

#import "PianoInstrument.h"
#import <AVFoundation/AVFoundation.h>

@implementation PianoInstrument {
    AVAudioEngine *_engine;
    AVAudioUnitSampler *_sampler;
    BOOL _noteMask[127];
}

- (instancetype)init {
    if ((self = [super init]) != nil) {
        _engine = [[AVAudioEngine alloc] init];
        _sampler = [[AVAudioUnitSampler alloc] init];
        
        [_engine attachNode:_sampler];
        [_engine connect:_sampler to:_engine.outputNode format:nil];
        
        NSURL *bundleURL = [[NSBundle mainBundle] URLForResource:@"Frameworks/PianoInstrumentPerformPlugin" withExtension:@"framework"];
        NSURL *instrumentURL = [[NSBundle bundleWithURL:bundleURL] URLForResource:@"rhodes" withExtension:@"sf2"];
        [_sampler loadInstrumentAtURL:instrumentURL error:NULL];
        
        [_engine startAndReturnError:NULL];
        
        memset(_noteMask, 0, sizeof(_noteMask));
    }
    return self;
}

- (void)noteOn:(uint8_t)note velocity:(uint8_t)velocity {
    if (!_noteMask[note]) {
        [_sampler startNote:note withVelocity:velocity onChannel:0];
        _noteMask[note] = YES;
    }
}

- (void)noteOff:(uint8_t)note {
    if (_noteMask[note]) {
        [_sampler stopNote:note onChannel:0];
        _noteMask[note] = NO;
    }
}

@end
